/*************************************************************************
*  Copyright (C) 2004 by Janek Kozicki                                   *
*  cosurgi@berlios.de                                                    *
*  Copyright (C) 2006 by Bruno Chareyre                                  *
*  bruno.chareyre@hmg.inpg.fr                                            *
*                                                                        *
*  This program is free software; it is licensed under the terms of the  *
*  GNU General Public License v2 or later. See file LICENSE for details. *
*************************************************************************/

#ifndef TRIAXIAL_STRESS_CONTROLLER_HPP
#define TRIAXIAL_STRESS_CONTROLLER_HPP

#include <yade/yade-core/DeusExMachina.hpp>
#include <yade/yade-lib-wm3-math/Math.hpp>
#include <yade/yade-lib-wm3-math/Vector3.hpp>

class PhysicalAction;
class MetaBody;
class PhysicalParameters;

class TriaxialStressController : public DeusExMachina 
{
	private :
		shared_ptr<PhysicalAction> actionParameterStiffnessMatrix;
		shared_ptr<PhysicalAction> actionParameterForce;
		int StiffnessMatrixClassIndex, ForceClassIndex;
		int wall_bottom, wall_top, wall_left, wall_right, wall_front, wall_back;
		
			
	public :
		//! CAUTION : interval must be equal to the interval of the StiffnessCounter, it is used here to check if stiffness has been computed 
		unsigned int interval;
		//! Defines the prescibed resultant force 
		Vector3r		force;	
		//! Stores the value of the translation at the previous time step
		Vector3r*		previoustranslation;
		//! The value of stiffness (updated according to interval) 
		Vector3r		stiffness;
		//! damping coefficient - damping=1 implies a "perfect" control of the resultant force
		Real			damping;
		//! maximum velocity (usefull to prevent explosions when stiffness is very low...) 
		Real			max_vel;
		
		int wall_bottom_id, wall_top_id, wall_left_id, wall_right_id, wall_front_id, wall_back_id;
		bool wall_bottom_activated, wall_top_activated, wall_left_activated, wall_right_activated, wall_front_activated, wall_back_activated;
		Real height, width, depth;
		Real thickness; // FIXME should retrieve "extents" of a InteractingBox
		Real sigma_iso;
		

		TriaxialStressController();
		virtual ~TriaxialStressController();
	
		virtual void applyCondition(Body*);
		void TriaxialStressController::controlStress(int wall, MetaBody* ncb, int id, Vector3r resultantForce, PhysicalParameters* p, Real wall_max_vel);
		
	
	protected :
		virtual void registerAttributes();
	REGISTER_CLASS_NAME(TriaxialStressController);
	REGISTER_BASE_CLASS_NAME(DeusExMachina);
};

REGISTER_SERIALIZABLE(TriaxialStressController,false);

#endif // TRIAXIAL_STRESS_CONTROLLER_HPP

